<?php
namespace oshwoo;

# object binding & access 
function osh() {
    return osh::get_instance();
}

/**
 * class osh
 *
 * main osh class, initializes the plugin
 *
 * @class       osh
 * @version     1.5
 * @author      alx359
 */
class osh {

    # properties|attributes
    public $show_donation_box = true;

    /**
     *  singleton lazy
     *  https://hardcorewp.com/2013/initializing-singleton-classes-used-in-wordpress-plugins/
     */
    private static $_instance;
    public  static function get_instance() {
        if( !isset( self::$_instance ) )
                    self::$_instance = new self;
             return self::$_instance;
    }

    #--> init
    function __construct() {
        $this->includes();
        $this->hooks();
    }

    private function includes() {
        require_once plugin_dir_path( namespace\FILE ) . 'includes/constants.php';
        require_once plugin_dir_path( namespace\FILE ) . 'includes/php_ext.php';
        require_once plugin_dir_path( namespace\FILE ) . 'includes/php_2js.php';
    }

    private function hooks() {
        # call register settings
        add_action( 'admin_init',                                 array( $this, 'register_plugin_settings' ) ); 
        # insert plugin submenus in WooC section
        add_action( 'admin_menu',                                 array( $this, 'add_submenus__wooc' ), 99 );
        # hide submenus
        add_filter( 'submenu_file',                               array( $this, 'hide_submenus__wooc' ) ); 
        # insert settings link in plugins page 
        add_action( 'plugin_action_links_'.namespace\PLUGIN_FILE, array( $this, 'add_settings__plugins_page' ), 10, 1 );
        # add browser-side scripts                                
        add_action( 'admin_enqueue_scripts',                      array( $this, 'enqueue_user_agent_scripts' ) );
        # insert new table columns                                
        # add a new column to WooCommerce > Orders                
        add_filter( 'manage_edit-shop_order_columns',             array( $this, 'add_order_table_column__orders_history' ), 20 );
        add_filter( 'manage_edit-shop_order_sortable_columns',    array( $this, 'add_order_table_column_sortable__orders_history' ) );
        add_action( 'pre_get_posts',                              array( $this, 'add_order_table_column_pre_get__orders_history' ) );
        add_action( 'manage_shop_order_posts_custom_column',      array( $this, 'add_order_table_column_custom__orders_history' ), 20, 2 );
        # add a new column to Users > All Users                   
        add_filter( 'manage_users_columns',                       array( $this, 'add_user_table_column__orders_history' ) );
        add_filter( 'manage_users_sortable_columns',              array( $this, 'add_user_table_column_sortable__orders_history' ) );
        add_filter( 'pre_get_users',                              array( $this, 'add_user_table_column_pre_get__orders_history' ) );
        add_filter( 'manage_users_custom_column',                 array( $this, 'add_user_table_column_custom__orders_history' ), 10, 3 );
        # add a metabox to Order edit page                        
        add_action( 'add_meta_boxes',                             array( $this, 'add_metabox__orders_history' ) );
        # load translations                                        
        load_plugin_textdomain( NS, false, namespace\DIR . 'i18n/languages' );
        # optional update of WC statuses colors                    
        add_action( 'admin_head',                                 array( $this, 'modify_wc_statuses_colors' ) );
        # update of WC widget statuses colors                      
        add_action( 'admin_footer',                               array( $this, 'modify_wc_widget_statuses_colors' ) );
        # add support for generic currency symbol within the report pages, and the WC widget
        if( !empty( get_option( 'oshwoo_multicurrency_symbol' ) ) ) {
            add_filter( 'woocommerce_currencies',                 NS.'\\add_currency' );
            add_filter( 'woocommerce_currency_symbol',            NS.'\\add_currency_symbol', 10, 2);
            # PHP_INT_MAX needed to override other multi-currency tweaking plugins, like Booster
            add_filter( 'woocommerce_currency_symbol',            NS.'\\add_currency_symbol_widget', PHP_INT_MAX, 1 ); 
        }
    }
    
    /**
     *  register plugin settings in wp_options
     */ 
    public function register_plugin_settings() {
        register_setting( 'oshwoo-settings', 'oshwoo_hx_guest',             array( 'default' => HX_GUEST ) );
        register_setting( 'oshwoo-settings', 'oshwoo_hx_aggregate',         array( 'default' => HX_AGGREGATE ) );
        register_setting( 'oshwoo-settings', 'oshwoo_hx_pending',           array( 'default' => HX_PENDING ) );
        register_setting( 'oshwoo-settings', 'oshwoo_hx_processing',        array( 'default' => HX_PROCESSING ) );
        register_setting( 'oshwoo-settings', 'oshwoo_hx_onhold',            array( 'default' => HX_ONHOLD ) );
        register_setting( 'oshwoo-settings', 'oshwoo_hx_completed',         array( 'default' => HX_COMPLETED ) );
        register_setting( 'oshwoo-settings', 'oshwoo_hx_cancelled',         array( 'default' => HX_CANCELLED ) );
        register_setting( 'oshwoo-settings', 'oshwoo_hx_refunded',          array( 'default' => HX_REFUNDED ) );
        register_setting( 'oshwoo-settings', 'oshwoo_hx_failed',            array( 'default' => HX_FAILED ) );
        register_setting( 'oshwoo-settings', 'oshwoo_hx_other',             array( 'default' => HX_OTHER ) );
        register_setting( 'oshwoo-settings', 'oshwoo_hx_text',              array( 'default' => HX_TEXT ) );
        register_setting( 'oshwoo-settings', 'oshwoo_hx_history',           array( 'default' => HX_HISTORY ) );
        register_setting( 'oshwoo-settings', 'oshwoo_wc_colors_update',     array( 'default' => '1') );
        register_setting( 'oshwoo-settings', 'oshwoo_multicurrency_symbol', array( 'default' => '' ) );
    }

    /**
     *  enqueue browser-side assets to only those pages this plugin works
     */  
    public function enqueue_user_agent_scripts() {
        $screen = get_current_screen();

        # [WC] for resource imports
        $wc_suffix  = \Automattic\Jetpack\Constants::is_true( 'SCRIPT_DEBUG' ) ? '' : '.min';
		$wc_version = \Automattic\Jetpack\Constants::get_constant( 'WC_VERSION' );

        //error_log( var_export( $screen->id, true ) );        
        switch( $screen->id  ) {

        case 'woocommerce_page_oshwoo-settings':
            # affects settings page only:
            #
            wp_enqueue_style ( 'wp-jquery-ui-dialog');
            wp_enqueue_script( 'jquery-ui-dialog' );
            wp_enqueue_script( 'jquery-csv',      namespace\URL . 'assets/js/jquery.csv.min.js', array('jquery'), '1.0.11', false );
            wp_enqueue_script( 'oshwoo-settings', namespace\URL . 'assets/js/settings.js', array('jquery', 'oshwoo-js', 'jquery-csv'), namespace\VERSION, false );
            wp_enqueue_script( 'oshwoo-themes',   namespace\URL . 'assets/js/themes.js',   array('jquery', 'oshwoo-js'),               namespace\VERSION, false );
            
        case 'woocommerce_page_oshwoo-order-history-t3':
        case 'woocommerce_page_oshwoo-order-history-t2': 
        case 'woocommerce_page_oshwoo-order-history-t1': 

            # [WC] notes bubbles & tooltip
            wp_enqueue_style ( 'woocommerce_admin_styles', WC()->plugin_url() . '/assets/css/admin.css', array(), $wc_version );
            # [WC] tooltip (activator in oshwoo.js)
            wp_enqueue_script( 'jquery-tiptip', WC()->plugin_url() . '/assets/js/jquery-tiptip/jquery.tipTip'. $wc_suffix. '.js', array( 'jquery' ), $wc_version, true );

            # affects custom pages only:
            #           
            wp_enqueue_style ( 'oshwoo-css', namespace\URL . 'assets/css/oshwoo.css', array(), namespace\VERSION, false );
            wp_enqueue_script( 'oshwoo-js',  namespace\URL . 'assets/js/oshwoo.js', array('jquery', 'postbox'), namespace\VERSION, false );
            
            # export php variables to js, and define js-specific localized strings
            # attaches to the 'oshwoo-js' handle
            # needs to be done right after the js enqueues for this to work
            php_2js();

        case 'edit-shop_order':
        case 'shop_order':
        case 'users':
            # affects all pages concerning this plugin:
            #
            wp_enqueue_style ( 'oshwoo-css', namespace\URL . 'assets/css/oshwoo.css' );
        }
    }

    /**
     *  add plugin submenus into the WooCommerce menu
     */ 
    public function add_submenus__wooc() {

        add_submenu_page('woocommerce', 
                      __('Customer History', NS), 
                      __('Customer History', NS), 
                         'manage_woocommerce', 
                         'oshwoo-order-history-t1', 
            array($this, 'add_order_history')
                        );
        # hidden
        add_submenu_page('woocommerce', 
                      __('Customer History', NS), 
                      __('Customer History', NS),  
                         'manage_woocommerce', 
                         'oshwoo-order-history-t2',
            array($this, 'add_order_history')
                        );
        # hidden
        add_submenu_page('woocommerce', 
                      __('Customer History', NS), 
                      __('Customer History', NS),  
                         'manage_woocommerce', 
                         'oshwoo-order-history-t3', 
            array($this, 'add_order_history')
                        );
        add_submenu_page('woocommerce', 
                      __('Status History Settings', NS), 
                      __('Status History Settings', NS), 
                         'manage_options', 
                         'oshwoo-settings', 
            array($this, 'add_settings')
                        );
    }

    /**
     *  hide plugin submenus
     *  https://stackoverflow.com/questions/3902760
     */
    public function hide_submenus__wooc( $submenu_file ) {
        global $plugin_page;
        
        # submenus to hide
        $hidden_submenus = array(
            'oshwoo-order-history-t2' => true,
            'oshwoo-order-history-t3' => true,
        );
        # Select another submenu item to highlight instead
        if ( $plugin_page && isset( $hidden_submenus[ $plugin_page ] ) ) {
            $submenu_file = 'oshwoo-order-history-t1';
        }
        # Hide the submenu.
        foreach ( $hidden_submenus as $submenu => $unused ) {
            remove_submenu_page( 'woocommerce', $submenu );
        }
        return $submenu_file;
    }
    
    public function add_order_history() {
        include( namespace\DIR . 'includes/admin/order-history.php' );
    }
    
    public function add_settings() {
        include( namespace\DIR . 'includes/admin/settings.php' );
    }

    /**
     *  add link to settings page in WP plugins page 
     *  https://neliosoftware.com/blog/how-to-add-a-link-to-your-settings-in-the-wordpress-plugin-list
     */
    public function add_settings__plugins_page( $links ) {
        # build and escape the URL
        $url = esc_url( add_query_arg(
            'page',
            'oshwoo-settings',
            get_admin_url() . 'admin.php'
        ) );
        # create the link
        $settings_link = "<a href='$url'>" . __('Settings', NS) . '</a>';
        # adds the link to the end of the array
        array_push(
            $links,
            $settings_link
        );
        return $links;
    }
    #<-- init

    #--> Orders page column builder
    /**
     *  Add column to Orders page
     */
    public function add_order_table_column__orders_history( $columns ) {
        $reordered_columns = array();
        # insert column to a specific location
        foreach( $columns as $key => $column ) {
            $reordered_columns[ $key ] = $column;
            if( $key == 'order_status' ) {
                # insert after 'Status' column
                $reordered_columns['oshwoo_orders_history'] = __('Orders history', NS);
            }
        }
        return $reordered_columns;
    }

    public function add_order_table_column_sortable__orders_history( $columns ) {
        $columns['oshwoo_orders_history'] 
               = 'oshwoo_orders_history';
        return $columns;
    }

    # fix sorting: https://stackoverflow.com/questions/48908754/
    function add_order_table_column_pre_get__orders_history( $query ) {
        
        //error_log( var_export( $query->get('orderby'), true ) );

        if ( 'oshwoo_orders_history' == $query->get('orderby') ) {
            $query->set('meta_key', 'oshwoo_aggregated');
            $query->set('orderby' , 'meta_value_num');
        }

    }
    
    public function add_order_table_column_custom__orders_history( $column, $order_id ) {
        
        # [optimization] get order data from the loop, instead of calling wc_get_order on every row
        # https://stackoverflow.com/questions/48908754
        global $post, $the_order;
        if ( empty( $the_order ) || $the_order->get_id() != $post->ID ) {
               $customer_order = wc_get_order( $post->ID );
        } else $customer_order = $the_order;
        
        //error_log( var_export( $post->ID, true ) );
        
        # add fix for admin page call with order_id that isn't an order
        if( !$customer_order ) return;
        
        # To discern later if the Order is from a registered Customer (by id) or a Guest (by email)
        $customer_id    = $customer_order->get_customer_id();
        $customer_email = $customer_order->get_billing_email();
        $orders         = $this->get_orders( $customer_id, $order_id );
        $is_guest       = $customer_id ? false : true;
        # Total number of Orders placed by this Customer
        $st_aggregated  = count( $orders );

        # for Orders history sorting in Orders table, set aggregated totals into wp_postmeta for each Order metadata 
        update_post_meta( $order_id, 'oshwoo_aggregated', $st_aggregated );
        
        # WC statuses counters init
        $wc_pending = $wc_processing = $wc_onhold = $wc_completed = $wc_refunded = $wc_cancelled = $wc_failed = $st_other = 0;
        
        if( $st_aggregated > 0 ) {
            foreach( $orders as $order ) { #1
                if( !isset( $order->post_status ) ) continue;
                switch( $order->post_status ) { #2
                case ST_PENDING:
                          $wc_pending++;
                          break;
                case ST_PROCESSING:
                          $wc_processing++;
                          break;
                case ST_ONHOLD:
                          $wc_onhold++;
                          break;
                case ST_COMPLETED:
                          $wc_completed++;
                          break;
                case ST_CANCELLED:
                          $wc_cancelled++;
                          break;
                case ST_REFUNDED:
                          $wc_refunded++;
                          break;
                case ST_FAILED:
                          $wc_failed++;
                          break;
                default:
                          $st_other++;
                          break;
                } #2
            } #1
        } #if
        
        switch( $column ) { #3
            
        case "oshwoo_orders_history": 

            # [WooCommerce > Orders] Render swatch statuses 
            if ( $is_guest ) { # Render Guest swatch
                echo $this->status_swatch_render( 'ST_GUEST', 'G', __('Order placed as Guest', NS) );
            }
            if( $st_aggregated > 0 ) {
            # Render status counters for registered customer, or a (returning) Guest
                echo $this->status_swatch_render( 'ST_AGGREGATE',  $st_aggregated, __('Total orders placed',    NS) );
                echo $this->status_swatch_render( 'ST_PENDING',    $wc_pending,    __('Orders pending payment', NS) );
                echo $this->status_swatch_render( 'ST_PROCESSING', $wc_processing, __('Orders processing',      NS) );
                echo $this->status_swatch_render( 'ST_ONHOLD',     $wc_onhold,     __('Orders on hold',         NS) );
                echo $this->status_swatch_render( 'ST_COMPLETED',  $wc_completed,  __('Orders completed',       NS) );
                echo $this->status_swatch_render( 'ST_CANCELLED',  $wc_cancelled,  __('Orders cancelled',       NS) );
                echo $this->status_swatch_render( 'ST_REFUNDED',   $wc_refunded,   __('Orders refunded',        NS) );
                echo $this->status_swatch_render( 'ST_FAILED',     $wc_failed,     __('Orders failed',          NS) );
                echo $this->status_swatch_render( 'ST_OTHER',      $st_other,      __('Orders in other status', NS) );
            }
            # Show history page link for orders > 1, to avoid cluttering
            if( $st_aggregated > 1 ) { 
                echo $this->history_swatch_render( $customer_id, $customer_email );
            }
        } #3
    }
    #<-- Orders page column builder

    #--> Users page column builder
    /**
     *  Add column to Users page
     */
    public function add_user_table_column__orders_history( $column ) {
        $column['oshwoo_orders_history'] = __('Orders History', NS);
        return $column;
    }
    
    public function add_user_table_column_sortable__orders_history( $columns ) {
        $columns['oshwoo_orders_history'] 
               = 'oshwoo_orders_history';
        return $columns;
    }
    
    public function add_user_table_column_pre_get__orders_history( $query ) {
        if ( 'oshwoo_orders_history' == $query->get('orderby') ) {
            $query->set('meta_key', 'oshwoo_aggregated');
            $query->set('orderby' , 'meta_value_num');
        }
    }
    
    public function add_user_table_column_custom__orders_history( $html, $column_name, $customer_id ) {
        switch( $column_name ) { #1
        case 'oshwoo_orders_history':
            
            $orders = $this->get_orders_from_customer( $customer_id );
            
            $st_aggregated = count( $orders );

            # for Orders history sorting in Users table, set aggregated totals into wp_usermeta for each User metadata 
            update_user_meta( $customer_id, 'oshwoo_aggregated', $st_aggregated );
 
            # WC statuses counters init
            $wc_pending = $wc_processing = $wc_onhold = $wc_completed = $wc_refunded = $wc_cancelled = $wc_failed = $st_other = 0;
            
            if( $st_aggregated > 0 ) {
                foreach( $orders as $order ) { #2
                    if( !isset( $order->post_status ) ) continue;
                    switch( $order->post_status ) { #3
                    case ST_PENDING:
                              $wc_pending++;
                              break;
                    case ST_PROCESSING:
                              $wc_processing++;
                              break;
                    case ST_ONHOLD:
                              $wc_onhold++;
                              break;
                    case ST_COMPLETED:
                              $wc_completed++;
                              break;
                    case ST_CANCELLED:
                              $wc_cancelled++;
                              break;
                    case ST_REFUNDED:
                              $wc_refunded++;
                              break;
                    case ST_FAILED:
                              $wc_failed++;
                              break;
                    default:
                              $st_other++;
                              break;
                    } #3
                } #2

                # [Users > All Users] Render swatch statuses for those WP users that have purchased something
                $html  = '';
                $html .= $this->status_swatch_render( 'ST_AGGREGATE',  $st_aggregated, __('Total orders placed',    NS), 1 );
                $html .= $this->status_swatch_render( 'ST_PENDING',    $wc_pending,    __('Orders pending payment', NS), 1 );
                $html .= $this->status_swatch_render( 'ST_PROCESSING', $wc_processing, __('Orders processing',      NS), 1 );
                $html .= $this->status_swatch_render( 'ST_ONHOLD',     $wc_onhold,     __('Orders on hold',         NS), 1 );
                $html .= $this->status_swatch_render( 'ST_COMPLETED',  $wc_completed,  __('Orders completed',       NS), 1 );
                $html .= $this->status_swatch_render( 'ST_CANCELLED',  $wc_cancelled,  __('Orders cancelled',       NS), 1 );
                $html .= $this->status_swatch_render( 'ST_REFUNDED',   $wc_refunded,   __('Orders refunded',        NS), 1 );
                $html .= $this->status_swatch_render( 'ST_FAILED',     $wc_failed,     __('Orders failed',          NS), 1 );
                $html .= $this->status_swatch_render( 'ST_OTHER',      $st_other,      __('Orders in other status', NS), 1 );
                # Always show history page link 
                $html .= $this->history_swatch_render( $customer_id, 0, 1 );
            } #if
        } #1
        return $html;
    }
    #<-- Users page column builder

    #--> Order page metabox builder
    public function add_metabox__orders_history() {
        add_meta_box( 'oshwoo_orders_history', __('Orders history', NS), array( $this, 'add_metabox_data__orders_history' ), 'shop_order', 'side', 'high' );
    }
    
    public function add_metabox_data__orders_history( $post_data, $metabox_data ) {
        $order_id       = $post_data->ID;
        $customer_order = wc_get_order( $order_id );
        $customer_id    = $customer_order->get_user_id();
        $customer_email = $customer_order->get_billing_email();
        $orders         = $this->get_orders( $customer_id, $order_id );
        $is_guest       = $customer_id ? false : true;
        # Total number of Orders placed by this Customer 
        $st_aggregated  = count( $orders );
        
        # WC statuses counters init
        $wc_pending = $wc_processing = $wc_onhold = $wc_completed = $wc_refunded = $wc_cancelled = $wc_failed = $st_other = 0;
        
        if( $st_aggregated > 0 ) {
            foreach( $orders as $order ) { #1
                if( !isset( $order->post_status ) ) continue;
                switch( $order->post_status ) { #2
                case ST_PENDING:
                          $wc_pending++;
                          break;
                case ST_PROCESSING:
                          $wc_processing++;
                          break;
                case ST_ONHOLD:
                          $wc_onhold++;
                          break;
                case ST_COMPLETED:
                          $wc_completed++;
                          break;
                case ST_CANCELLED:
                          $wc_cancelled++;
                          break;
                case ST_REFUNDED:
                          $wc_refunded++;
                          break;
                case ST_FAILED:
                          $wc_failed++;
                          break;
                default:
                          $st_other++;
                          break;
                } #2
            } #1
        } #if
        
        # [Edit Order details] Render swatch statuses
        echo '<div id="oshwoo-status-holder">'.PHP_EOL;
        if ( $is_guest ) { # Render Guest swatch
            echo $this->status_swatch_render( 'ST_GUEST', 'G', __('Order placed as Guest', NS) );
        }
        if( $st_aggregated > 0 ) {
            # Render status counters for registered customer, or a (returning) Guest
            echo $this->status_swatch_render( 'ST_AGGREGATE',  $st_aggregated, __('Total orders placed',    NS) );
            echo $this->status_swatch_render( 'ST_PENDING',    $wc_pending,    __('Orders pending payment', NS) );
            echo $this->status_swatch_render( 'ST_PROCESSING', $wc_processing, __('Orders processing',      NS) );
            echo $this->status_swatch_render( 'ST_ONHOLD',     $wc_onhold,     __('Orders on hold',         NS) );
            echo $this->status_swatch_render( 'ST_COMPLETED',  $wc_completed,  __('Orders completed',       NS) );
            echo $this->status_swatch_render( 'ST_CANCELLED',  $wc_cancelled,  __('Orders cancelled',       NS) );
            echo $this->status_swatch_render( 'ST_REFUNDED',   $wc_refunded,   __('Orders refunded',        NS) );
            echo $this->status_swatch_render( 'ST_FAILED',     $wc_failed,     __('Orders failed',          NS) );
            echo $this->status_swatch_render( 'ST_OTHER',      $st_other,      __('Orders in other status', NS) );
        }
        # Show history page link for orders > 1, to avoid cluttering
        if( $st_aggregated > 1 ) {
            echo $this->history_swatch_render( $customer_id, $customer_email );
        }
        echo '</div>' . PHP_EOL;
    }
    #<-- Order page metabox builder

    #--> html renders
    public function status_swatch_render( $status_str, $text, $title, $btitle=false ) {
        $status_color = $this->get_color( $status_str );
        $text_color   = $this->get_color('HX_TEXT');
        $title_type   = $btitle ? 'title' : 'data-tip';

        # Show aggregate swatch only for orders > 1, to avoid cluttering
        if( $status_str == 'ST_AGGREGATE' && $text == 1 ) return;

        return $text ? '<span data-status="' . $text . '" class="oshwoo swatch tips" '
                     . 'style="color: ' . $text_color . '; background-color: ' . $status_color . ';" ' 
                     . $title_type . '="' . $title . '">'
                     . $text . '</span>' . PHP_EOL : '';
    }

    public function history_swatch_render( $userid, $userem = false, $btitle=false ) {
        $history_color = $this->get_color('HX_HISTORY');
        $text_color    = $this->get_color('HX_TEXT');
        $title_type    = $btitle ? 'title' : 'data-tip';
        
        if( $userid ): 
            $arg['userid'] = $userid;
        elseif( $userem ): 
            $arg['userem'] = urlencode( $userem );
        endif;

        return '<a href="' . esc_url( add_query_arg( $arg, site_url( '/wp-admin/admin.php?page=oshwoo-order-history-t1' ) ) )
             . '"><span class="oshwoo swatch icon tips" '
             . 'style="color: ' . $text_color . '; background-color: ' . $history_color . ';" '
             . $title_type . '="Orders History"></span></a>' . PHP_EOL;
    }

    public function history_button_render( $userid, $userem, $tab, $type, $title ) {
        if( $userid ): 
            $arg['userid'] = $userid;
        elseif( $userem ): 
            $arg['userem'] = urlencode( $userem );
        endif;
            $arg['type']   = $type;
            $arg['action'] = 'download';
        
        return '<a href="'. esc_url( add_query_arg( $arg, site_url('/wp-admin/admin.php?page=oshwoo-order-history-'.$tab) ) ) 
              .'" class="button-primary" style="cursor: pointer;" target="_blank">' . $title . '</a>' . PHP_EOL;
    }

    public function settings_button_render( $action, $title ) {
        $html = '';
        switch( $action ) {
        case 'import':
            $html .= '<input type="file" id="oshwoo_import_settings" name="oshwoo_import_settings" style="display:none" onclick="this.value=null" accept=".csv">' . PHP_EOL;
            $html .= '<label for="oshwoo_import_settings" class="button-primary" style="vertical-align:top">' .$title. '</label>' . PHP_EOL;
            break;
        case 'export':
            $arg['action'] = $action;
            $html .= '<a href="'. esc_url( add_query_arg( $arg, site_url('/wp-admin/admin.php?page=oshwoo-settings') ) ) 
                    .'" class="button-primary" style="cursor: pointer;" target="_blank">' .$title. '</a>' . PHP_EOL;
            break;
        }
        return $html;
    }

    public function create_page_tabs( $tab_active = 't1', $userid, $userem = false ) {
        if( $userid ): 
            $arg['userid'] = $userid;
        elseif( $userem ): 
            $arg['userem'] = urlencode( $userem );
        endif;
        
        $tabs = array(
            't1' => __('Orders', NS), 
            't2' => __('Products', NS),
            't3' => __('Notes', NS),
        );
        $html = '<h2 class="nav-tab-wrapper">';
        foreach( $tabs as $tab => $name ) {
            $class = ( $tab == $tab_active ) ? 'nav-tab-active' : '';
            $html .= '<a href="' . esc_url( add_query_arg( $arg, site_url( '/wp-admin/admin.php?page=oshwoo-order-history-'. $tab ) ) )
                  .  '" class="nav-tab ' . $class . '">' . $name . '</a>';
        }
        $html .= '</h2>' . PHP_EOL;
        echo $html;
    }

    /**
     *  Process settings option to update WC color statuses 
     */
    public function modify_wc_statuses_colors() {
        global $pagenow, $post;

        if( $pagenow != 'edit.php' ) return;
        if( !$post || get_post_type( $post->ID ) != 'shop_order' ) return;
        if( !get_option( 'oshwoo_wc_colors_update' ) ) return;
        
        echo '<style>'  . PHP_EOL;
        foreach( wc_get_order_statuses() as $status => $value ) {
             # normalize WC status name to get HEX color for it
             $hex_name = 'HX_' . strtoupper( str_replace( '-', '', substr( $status, 3 ) ) );
             //error_log( var_export( $hex_name, true ) );
             echo '.order-status.status-' . status_nwc( $status ) . PHP_EOL;
             # check if a WC status name was passed, and normalize it to extract our HEX counterpart 
             echo '{ color: ' .  $this->get_color('HX_TEXT') . '; background-color: ' . $this->get_color( $hex_name ) . '}' . PHP_EOL;
             
        }
        echo '</style>' . PHP_EOL;
    }
    
    /**
     *  Process settings option to also update WC color widget statuses 
     */
    public function modify_wc_widget_statuses_colors() {
        global $pagenow;

        if( $pagenow != 'index.php' ) return;
        if( !get_option( 'oshwoo_wc_colors_update' ) ) return;

        # unicode to octal, e.g. U+1F680 = \1F680
        ?>
        <style type="text/css">
        /* oshwoo: custom color for 'processing' status */
        #woocommerce_dashboard_status .wc_status_list li.processing-orders a::before
        { color:<?php echo $this->get_color('HX_PROCESSING') ?>; }
        /* oshwoo: custom color for 'on-hold' status */
        #woocommerce_dashboard_status .wc_status_list li.on-hold-orders a::before
        { color:<?php echo $this->get_color('HX_ONHOLD') ?>; }
        /* oshwoo: custom icon for'Pending' status */
        #woocommerce_dashboard_status .wc_status_list li.pending a::before
        { content:'\e016'; color:<?php echo $this->get_color('HX_PENDING') ?>; }
        /* oshwoo: custom icon for'Best Seller this month' */
        #woocommerce_dashboard_status .wc_status_list li.best-seller-this-month a::before
        { content:'\1F680'; color:#000; }
        </style>
        <?php
    }

    #<-- html renders

    #--> DB access    
 
    public function get_orders( &$customer_id, $order_id = false ) {
        
        # Handle of edge & manual cases:
        #
        # If a Guest Order gets placed by a (non-logged) returning Customer, 
        # we attempt to find customer_id from previous orders, 
        # implying at least one might be placed as a non-Guest 
        $orders = $customer_id ? $this->get_orders_from_customer( $customer_id )  
                               # Get other Orders from current Order data
                               # (may also update customer_id inside)
                               : $this->get_orders_from_order( $customer_id, $order_id );

        # handle cases where order was created manually by an admin/shop manager
        if( !$orders ) {
            
            # Order created manually. Single Guest order with yet incomplete data (no billing email)
            if( !$customer_id ) {
                # make result countable 
                $orders[0] = get_object_vars( get_post( $order_id ) );
            }
            # Order created manually, by admin/manager
            if( $customer_id ) {
                $roles = get_userdata( $customer_id )->roles;
                if ( in_array( 'administrator', $roles, true ) 
                ||   in_array( 'shop_manager',  $roles, true ) ) {
                    # recursively look at the Order as Guest
                    $orders = $this->get_orders( $customer_id = 0, $order_id );
                    # exclude admin/managers from messing with the Guest status 
                    $customer_id = 0;
                }
            }
        }
        //error_log( var_export( count( $orders ), true ) );        
        return ( $orders ? $orders : array() );
    }

    /**
     *  Retrieves lightweight post objects, sufficient for the purpose of counting order statuses
     */
    public function get_orders_from_customer( $customer_id, $customer_email = false, $status = false, $limit = -1 ) {

        # Need id or email to get orders
        if( !$customer_id && !$customer_email ) return;
        
        # Get email from given Customer, to look up for non-logged (Guest) orders 
        if( $customer_id && !$customer_email ) {
            $customer       = get_user_by( 'id', absint( $customer_id ) );
            $customer_email = $customer->user_email;
        }

        # Get all Orders as Posts for a given Customer 
        # Based on: https://www.skyverge.com/blog/get-all-woocommerce-orders-for-a-customer/
        $orders = get_posts(array(
            'numberposts' => $limit,
            'orderby'     => 'date',
            'order'       => 'DESC',
            'post_type'   => array('shop_order'),
            # get_posts expects the wc-prefixed status
            'post_status' => ( $status ? $status : array_keys( wc_get_order_statuses() ) ),
            'meta_key'    => '_customer_user',
            'meta_value'  => $customer_id,
            'meta_key'    => '_billing_email',
            'meta_value'  => $customer_email,
        ));

        //error_log( var_export( $customer_email, true ) );
        return $orders;
    }
    
    /**
     *  Same as get_orders_from_customer() but quite more resource intensive, as it retrieves the complete order objects 
     *  Only required in order-history page for detailed views
     */
    public function get_orders_from_customer_wc( $customer_id, $customer_email = false, $status = false, $limit = -1 ) {
        
        # Need id or email to get orders
        if( !$customer_id && !$customer_email ) return;
        
        # Find orders by email to look up for related Guest orders 
        if(  $customer_id && !$customer_email ) {
             $customer       = get_user_by( 'id', absint( $customer_id ) );
             $customer_email = $customer->user_email;
        }
        # Get all Orders for a given Customer 
        $query = new \WC_Order_Query(array(
            'limit'      => $limit,
            'orderby'    => 'date',
            'order'      => 'DESC',
            'type'       => array('shop_order'),
            'return'     => 'objects',
            # WC_Order_Query expects the non-prefixed status
            'status'     => ( $status ? status_nwc($status) : array_keys( $this->get_order_statuses() ) ),
            'meta_key'   => '_customer_user',
            'meta_value' => $customer_id,
            'meta_key'   => '_billing_email',
            'meta_value' => $customer_email,
        ));
        $orders = $query->get_orders();
        
        //error_log( var_export( $orders, true ) );
        return $orders;
    }

    public function get_orders_from_order( &$customer_id, $order_id ) {
        # if the query comes from a Guest Order that's also getting other (email) matching results, 
        # attempt to find if there's a registered Customer placing them
        if( $order_id ) {
            $customer_email = get_post_meta( $order_id, '_billing_email', true );
            $orders = $this->get_orders_from_customer( 0, $customer_email );
            if( !$orders ) return;
            if( !$customer_id ) {
                # attempt to find id from previous orders
                foreach( $orders as $order ) {
                    $user_id = get_post_meta( $order->ID, '_customer_user', true );
                    if( $user_id ) {
                        # update id implicitly in caller
                        $customer_id = $user_id;
                        break;
                    }
                }
            }
            //error_log( var_export( $customer_id, true ) );
            return $orders;
        }
    }
    #<-- DB access

    #--> class helpers
    
    /**
     *  Get valid statuses for query types that do not support wc prefixes
     *  https://woocommerce.wp-a2z.org/oik_api/wc_api_ordersget_order_statuses/
     */
    public static function get_order_statuses() {
        $order_statuses = array();        
        foreach( wc_get_order_statuses() as $slug => $name ) {
            $order_statuses[ str_replace( 'wc-', '', $slug ) ] = $name;
        }
        return $order_statuses;
    }
    
    /**
     *  Get customized (wp_options) or default (const) colors
     */
    public function get_color( $str_name ) {
        # extract hex const from paired status counterpart (st/hx pair must exists)
        $str_name = strtoupper( $str_name );
            if( strpos( $str_name, 'ST_' ) !== false ) $hex_name = 'HX_' . substr( $str_name, 3);
        elseif( strpos( $str_name, 'HX_' ) !== false ) $hex_name =                 $str_name;
        else return;
        //error_log( var_export( "{$hex_name}=" . defined(__NAMESPACE__ . "\\{$hex_name}"), true ) );
        # get value from global const
        if( !defined( __NAMESPACE__ . "\\{$hex_name}" ) || !( $hex_val = constant( __NAMESPACE__ . "\\{$hex_name}" ) ) ) return;
        return empty( get_option( 'oshwoo_' . strtolower( $hex_name ) ) ) ? $hex_val 
                    : get_option( 'oshwoo_' . strtolower( $hex_name ) );
    }
    #<-- class helpers

} # osh class
